// copied from https://www.https://www.gribblelab.org/CBootCamp/A2_Parallel_Programming_in_C.html#org69fa9a8
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mpi.h>

#define MASTER 0         // task ID of master task
#define NDARTS 1000      // # dart throws per round
#define NROUNDS 10     // # of rounds of dart throwing

// our function for throwing darts and estimating pi
double dartboard(int ndarts)
{
  double x, y, r, pi; 
  int n, hits;

  hits=0; // initialize the number of hits in the circle.
  // throw darts
  for (n = 1; n <= ndarts; n++)  {
    // (x,y) are random between -1 and 1
    r = (double)random()/(double)RAND_MAX;
    x = (2.0 * r) - 1.0;
    r = (double)random()/(double)RAND_MAX;
    y = (2.0 * r) - 1.0;
    // if our random dart landed inside the unit circle, increment the score
    if (sqrt((x*x) + (y*y)) <= 1.0) {
      hits++;
    }
  }

  // estimate pi
  pi = (double)hits / (double)ndarts;
  return(pi);
} 

// the main program
int main (int argc, char *argv[])
{
  double my_pi, pi_sum, pi_est, mean_pi, err;
  int task_id, n_tasks, rc, i;
//  MPI_Status status;

  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&n_tasks);
  MPI_Comm_rank(MPI_COMM_WORLD,&task_id);
//  printf("I am node %d of %d\n", task_id, n_tasks);

  // different seed for random number generator for each task
  srandom (task_id);

  mean_pi = 0.0;
  for (i=0; i<NROUNDS; i++) {
    // all tasks will execute dartboard() to calculate their own estimate of pi
    my_pi = dartboard(NDARTS);
//   printf  (" node %d completed darts mypi=%2.12f\n",task_id,my_pi);
//    MPI_Wait(); 
   // now we use MPI_Reduce() to sum values of my_pi across all tasks
    // the master process (id=MASTER) will store the accumulated value
    // in pi_sum. We tell MPI_Reduce() to sum by passing it
    // the MPI_SUM value (define in mpi.h)
    MPI_Reduce(&my_pi, &pi_sum, 1, MPI_DOUBLE, MPI_SUM,MASTER, MPI_COMM_WORLD);

    // now, IF WE ARE THE MASTER process, we will compute the mean
    if (task_id == MASTER) {
      mean_pi =4.00000000000000* pi_sum / n_tasks;
//      mean_pi = ( (mean_pi * (i-1)) + pi_est ) / (i); // running average
      err = mean_pi - 3.14159265358979323846;
      printf("%d throws: mean_pi %2.12f: error %2.12f\n",
       (NDARTS * (i + 1)), mean_pi, err);
    }
  }
  if (task_id == MASTER) 
    printf ("PS, the real value of pi is about 3.14159265358979323846\n");

  MPI_Finalize();
  return 0;
}
